package cn.com.infosec.crypto.modes;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.params.ParametersWithIV;

public class GOFBBlockCipher implements BlockCipher {
   static final int C1 = 16843012;
   static final int C2 = 16843009;
   private byte[] IV;
   private byte[] ofbV;
   private byte[] ofbOutV;
   private final int blockSize;
   private final BlockCipher cipher;
   boolean firstStep = true;
   int N3;
   int N4;

   public GOFBBlockCipher(BlockCipher cipher) {
      this.cipher = cipher;
      if ((this.blockSize = cipher.getBlockSize()) == 8) {
         this.IV = new byte[cipher.getBlockSize()];
         this.ofbV = new byte[cipher.getBlockSize()];
         this.ofbOutV = new byte[cipher.getBlockSize()];
      } else {
         throw new IllegalArgumentException("GCTR only for 64 bit block ciphers");
      }
   }

   private int bytesToint(byte[] in, int inOff) {
      return (in[inOff + 3] << 24 & -16777216) + (in[inOff + 2] << 16 & 16711680) + (in[inOff + 1] << 8 & '\uff00') + (in[inOff] & 255);
   }

   private void intTobytes(int num, byte[] out, int outOff) {
      int var4 = outOff + 3;
      out[var4] = (byte)(num >>> 24);
      var4 = outOff + 2;
      out[var4] = (byte)(num >>> 16);
      var4 = outOff + 1;
      out[var4] = (byte)(num >>> 8);
      out[outOff] = (byte)num;
   }

   public BlockCipher getUnderlyingCipher() {
      return this.cipher;
   }

   public void init(boolean encrypting, CipherParameters params) {
      this.firstStep = true;
      this.N3 = 0;
      this.N4 = 0;
      if (params instanceof ParametersWithIV) {
         byte[] var3;
         ParametersWithIV encrypting1;
         byte[] params1;
         if ((params1 = (encrypting1 = (ParametersWithIV)params).getIV()).length < (var3 = this.IV).length) {
            int var4 = var3.length - params1.length;
            int var5 = params1.length;
            System.arraycopy(params1, 0, var3, var4, var5);

            byte[] var10;
            for(int var9 = 0; var9 < (var10 = this.IV).length - params1.length; ++var9) {
               var10[var9] = 0;
            }
         } else {
            int params2 = var3.length;
            System.arraycopy(params1, 0, var3, 0, params2);
         }

         this.reset();
         this.cipher.init(true, encrypting1.getParameters());
      } else {
         this.reset();
         this.cipher.init(true, params);
      }

   }

   public String getAlgorithmName() {
      return this.cipher.getAlgorithmName() + "/GCTR";
   }

   public int getBlockSize() {
      return this.blockSize;
   }

   public int processBlock(byte[] in, int inOff, byte[] out, int outOff) {
      int var5;
      if (inOff + (var5 = this.blockSize) > in.length) {
         throw new DataLengthException("input buffer too short");
      } else if (outOff + var5 > out.length) {
         throw new DataLengthException("output buffer too short");
      } else {
         byte[] var6;
         byte[] var8;
         if (this.firstStep) {
            this.firstStep = false;
            var8 = this.ofbV;
            var6 = this.ofbOutV;
            this.cipher.processBlock(var8, 0, var6, 0);
            this.N3 = this.bytesToint(this.ofbOutV, 0);
            this.N4 = this.bytesToint(this.ofbOutV, 4);
         }

         this.N3 = var5 = this.N3 + 16843009;
         this.N4 += 16843012;
         this.intTobytes(var5, this.ofbV, 0);
         var5 = this.N4;
         this.intTobytes(var5, this.ofbV, 4);
         var8 = this.ofbV;
         var6 = this.ofbOutV;
         this.cipher.processBlock(var8, 0, var6, 0);

         int var9;
         for(var5 = 0; var5 < (var9 = this.blockSize); ++var5) {
            out[outOff + var5] = (byte)(this.ofbOutV[var5] ^ in[inOff + var5]);
         }

         byte[] var10002 = in = this.ofbV;
         inOff = in.length - var9;
         System.arraycopy(var10002, var9, in, 0, inOff);
         int var7;
         inOff = (in = this.ofbV).length - (var7 = this.blockSize);
         System.arraycopy(this.ofbOutV, 0, in, inOff, var7);
         return this.blockSize;
      }
   }

   public void reset() {
      GOFBBlockCipher var10000 = this;
      byte[] var10001 = this.IV;
      byte[] var2 = this.ofbV;
      int var1 = var10001.length;
      System.arraycopy(var10001, 0, var2, 0, var1);
      var10000.cipher.reset();
   }
}
